#!/usr/bin/env ruby
  
# NSE nmap XML to CSV Converter (Example)
#
# An example on how to extract data from nmap Output files and put them into
# a csv file for analysis.
#
# This is a demonstration script and lacks a lot of bells and whistles.
# Apart from being lazy, I wanted this to be as short and compact as possible
# to make the code read- and understandable in a short amount of time.
#
# Cauution: Especially error handling is rarely existant at all in this example.
#
# = Homepage
#
# http://www.scip.ch
#
# = Author
#
# Stefan Friedli (stfr@scip.ch)
#
# The nmap parser library was written bei Kris Katterjohn
# Kris Katterjohn (katterjohn@gmail.com)
#
# = License
#
# Copyright (c) 2008-2010 scip AG
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

  
  # Standard Libs
  require 'rubygems'
  require 'commandline'
  
  # Nmap Library
  require 'lib/parser'
  
  class App < CommandLine::Application
	def initialize
		author    "Stefan Friedli <stfr@scip.ch>"
		copyright "scip AG - 2010"
		options :help, :debug
		option  :names => %w(--scan-file -i),
				:opt_found => get_args,
				:opt_not_found   => lambda { false },
				:opt_description => "nmap XML output file to parse",
				:arity => [1,1],
				:arg_description => "scan file"
				
				option  :names => %w(--target-file -t),
				:opt_found => get_args,
				:opt_not_found   => lambda { File.expand_path(Dir.pwd) + "/results.csv" },	
				:opt_description => "The target file to create. If none is supplied, output will be written to current directory (results.csv)",
				:arity => [1,-1],
				:arg_description => "target file"
				
    end

    def main
		puts "nmapxml2csv.rb 1.0 launched..."
	
		@ifile 		= opt["--scan-file"]			# Results in Array or String depending on the amount of parameters submitted.
		@ofile  		= opt["--target-file"]		# Contains the target file.
		@hosts = Array.new							# This array will hold our hosts as well as any underlying information
		
		if not @ifile 								# Check if file parameter is existant.
			puts "You need to supply a file to parse. Exiting"
			exit
		end
		
		# Parse File
		parser = Nmap::Parser.parsefile(@ifile)
		if parser
			parser.hosts("up") do |h|; @hosts.push(h); end	# We are only interested in the hosts that nmap confirmed to exist
			
			@hosts.each { |host|								# See host.inspect for the whole host object. You can output whatever you want, we'll stick with hostname/ip, a port dump and the script output here
				
				# Get Hostname and IP
				o_hostname = host.hostname.nil? ? "" : host.hostname	# Sometimes, you won't have hostnames in your result (for instance when using the -n parameter), so we need to catch this
				o_ip	   = host.addr
				
				o_sep	   = ";"
				
				# Get Host Scripts
				# Nmap separates host and port scripts. In this case, we just want to dump out all script results, so we combine those into the array scriptdump
				scriptdump = host.scripts
				
				# Build a string listing all open ports (tcp/udp)
				# Additional: Get the port scripts into the scriptdump Array
				# Check out the documentation for getports to find out about the filter possibilities
				o_plist = String.new
				host.getports(:tcp, "open") {|p| o_plist += "tcp/#{p.num}, "; p.scripts.each{|s| scriptdump.push(s)}}
				host.getports(:udp, "open") {|p| o_plist += "udp/#{p.num}, "; p.scripts.each{|s| scriptdump.push(s)}}
				o_plist.chop!.chop! # Kill the last separator (", ") using this ugly bit of code that sounds surprisingly funny if you read it aloud.
				
				# Unwrap scip inline Formatting and write this in a CSV file
				
				fh = File.open(@ofile, "wb")
				
				scriptdump.each {|s|
					tsID				= /sID\{(.*?)\}(,|;)/m.match(s.output)[1]
					tfilename			= s.id
					ttimestamp			= /sTimestamp\{(.*?)\}(,|;)/m.match(s.output)[1]
					taccuracy			= /sAccuracy\{(.*?)\}(,|;)/m.match(s.output)[1]
					ttype				= /sTesttype\{(.*?)\}(,|;)/m.match(s.output)[1]
					tsource				= /sTestsource\{(.*?)\}(,|;)/m.match(s.output)[1]	
					tversion			= /sVersion\{(.*?)\}(,|;)/m.match(s.output)[1]	
					tresult				= /sOutput\{(.*?)\}(,|;)/m.match(s.output)[1].gsub('"', '""')
					
					
					
					# Feel free to alternate the script output here
					output =  "\"#{o_ip}\"#{o_sep}\"#{o_hostname}\"#{o_sep}\"#{o_plist}\"#{o_sep}\"#{tsID}\"#{o_sep}\"#{tfilename}\"#{o_sep}\"#{ttimestamp}\"#{o_sep}"
					output += "\"#{taccuracy}\"#{o_sep}\"#{ttype}\"#{o_sep}\"#{tsource}\"#{o_sep}\"#{tversion}\"#{o_sep}\"#{tresult}\""
					
					fh.puts(output)
				}
				
				fh.close
				puts "Finished... see #{@ofile} for results."

			}
		end
		
	end

  end#class App
